%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%
%%% A simulation example of the fixed design where
%%% the predictor matrix X is the square root of a tri-diagonal matrix.
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear;
%% Generate X
p = 5000;
%%% Construct a tri-diagonal matrix
rho = 0.4;
II = [1:p, 1:(p-1), 2:p];
JJ = [1:p, 2:p, 1:(p-1)];
VV = [ones(1, p), rho*repmat([-1, 1, 1], 1, (p-2)/3), -rho, rho*repmat([-1, 1, 1], 1, (p-2)/3), -rho];
Omega = sparse(II, JJ, VV);
shuffler = randperm(p);
Omega = Omega(shuffler, shuffler);
%%% Use the square root as X
OmegaRoot = full( chol(Omega));
n = p;
X = OmegaRoot;

%% Generate the signal beta

v = 0.32;
r = 2;
tau = sqrt(2*r*log(p))
sp = p^(1-v)

%%% generate the support of the signals
sigseed = rand(p, 1);
signal = (abs(sigseed-0.5) < p^(-v)/2);
sumsig = sum(signal)
%%% generate the signs of the signals
signbeta = 2*(randi(2, [p, 1])-1.5).*signal;
%%% generate signals with different strength.
tt = 1;
uun = randn(p, tt).*(rand(p, 1) < 0.2);
uun2 = (sum(uun.^2,2)/tt/6);
beta = tau*(1+uun2).*signbeta;

%% generate y
z = randn(n, 1);
y = X*beta + z;


%% Optional block: Lasso procedure for this problem. 
% %%% you can compare our result with the lasso's.
% qV=v+(r-v).*(1:50)./50;
% hamLassoV=zeros(1,50);
% for jjj=1:50
% qlasso=qV(jjj);
% lambdaLasso =  sqrt(2*qlasso*log(p));
% lambdaLasso = lambdaLasso/sqrt(n); % rescaling needed by glmnet
% options.lambda = lambdaLasso;
% options.type = 'naive';
% options = glmnetSet(options);
% % run lasso
% fitLasso = glmnet(X,y,'gaussian', options);
% estimateLasso = fitLasso.beta;
% hamLassoV(jjj)=sum(signbeta~= sign(estimateLasso));
% end
% %%% We pick the best lasso can do in the above 50 runs.
% hamLasso=min(hamLassoV)

%% prepare UPS and GS
%%% prepare for the graphlet screening
yt = X'*y;
neighbor = sparse((Omega ~= 0));
neighbor = neighbor-sparse(1:p, 1:p, 1);
nm = 3;
CGA = FindAllCG(neighbor, nm);

lambda = sqrt(2*v*log(p));
uu = tau;

%% UPS procedure
%%% the screening step, 
%%% when nm = 1 in the screening step, the procedure is UPS procedure.
survivor = ScreeningStep(yt, Omega, CGA, 1, v, r);
sum(survivor)
sserr = sum(survivor < signal)
%%% the cleaning step
betaUPS = CleaningStep(survivor, yt, Omega, lambda, uu);
hamUPS = sum(sign(betaUPS) ~= sign(beta))

%% GS procedure
%%% the screening step
survivor = ScreeningStep(yt, Omega, CGA, nm, v, r);
sum(survivor)
sserr = sum(survivor < signal)
%%% the cleaning step
betaGS = CleaningStep(survivor, yt, Omega, lambda, uu);
hamGS = sum(sign(betaGS) ~= sign(beta))

sum(betaGS ~= 0)





